@@ -19,7 +19,7 @@ from neurst.data.text import build_tokenizer
 from neurst.data.text.vocab import PaddingMode, Vocab
 
 
-@register_data_pipeline("simple_text")
+@register_data_pipeline(["simple_text", "TranscriptDataPipeline", "transcript_data_pipeline"])
 class TextDataPipeline(DataPipeline, Vocab):
 
     def __init__(self,
@@ -30,6 +30,10 @@ class TextDataPipeline(DataPipeline, Vocab):
                  subtokenizer_codes=None,
                  glossaries=None,
                  reverse_sequence=False,
+                 bos_id=None,
+                 eos_id=None,
+                 unk_id=None,
+                 pad_id=None,
                  **kwargs):
         """ Initializes the data pipeline for text data.
 
@@ -64,14 +68,16 @@ class TextDataPipeline(DataPipeline, Vocab):
             tokens = Vocab.load_tokens(tokens=vocab_path)
         else:
             tokens = Vocab.load_tokens(vocab_path=vocab_path)
-        unk_token = Vocab.get_unique(tokens, "<UNK>")
-        bos_token = Vocab.get_unique(tokens, "<SEQ_BEG>")
-        eos_token = Vocab.get_unique(tokens, "<SEQ_END>")
+        unk_token = Vocab.get_unique(tokens, "<UNK>") if unk_id is None else tokens[unk_id]
+        bos_token = Vocab.get_unique(tokens, "<SEQ_BEG>") if bos_id is None else tokens[bos_id]
+        eos_token = Vocab.get_unique(tokens, "<SEQ_END>") if eos_id is None else tokens[eos_id]
+        pad_token = eos_token if pad_id is None else tokens[pad_id]
         assert unk_token != bos_token != eos_token
-        Vocab.__init__(self, tokens, [unk_token, bos_token, eos_token], lowercase=False)
+        Vocab.__init__(self, tokens, [unk_token, bos_token, eos_token, pad_token], lowercase=False)
         self._eos_id = Vocab.map_token_to_id(self, eos_token)
         self._bos_id = Vocab.map_token_to_id(self, bos_token)
         self._unk_id = Vocab.map_token_to_id(self, unk_token)
+        self._pad_id = Vocab.map_token_to_id(self, pad_token)
 
     @property
     def meta(self):
@@ -82,10 +88,27 @@ class TextDataPipeline(DataPipeline, Vocab):
             "bos_id": self._bos_id,
             "unk_id": self._unk_id,
             "pad_id": self._eos_id,
-            "padding_mode": PaddingMode.EOS_AS_PADDING
+            "padding_mode": (PaddingMode.EOS_AS_PADDING
+                             if self._eos_id == self._pad_id else PaddingMode.DEFAULT)
         }
 
-    def process(self, input, is_processed=False):
+    def preprocess(self, input):
+        input = DataPipeline.text_pre_normalize(self, self._language, input, is_processed=False)
+        if self._tokenizer:
+            input = self._tokenizer.tokenize(input, return_str=True)
+        if self._subtokenizer:
+            input = self._subtokenizer.tokenize(input, return_str=True)
+        return input
+
+    def postprocess(self, input):
+        output = input
+        if self._subtokenizer is not None:
+            output = self._subtokenizer.detokenize(output, return_str=True)
+        if self._tokenizer is not None:
+            output = self._tokenizer.detokenize(output, return_str=True)
+        return output
+
+    def encode(self, input, is_processed=False):
         """ Process one data sample.
 
         Args:
@@ -96,10 +119,7 @@ class TextDataPipeline(DataPipeline, Vocab):
             A list of generated token IDs.
         """
         if not is_processed:
-            if self._tokenizer:
-                input = self._tokenizer.tokenize(input)
-            if self._subtokenizer:
-                input = self._subtokenizer.tokenize(input, return_str=False)
+            input = self.preprocess(input)
         if isinstance(input, str):
             input = input.split()
         token_ids = Vocab.map_token_to_id(self, input, unknown_default=self._unk_id)
@@ -107,7 +127,7 @@ class TextDataPipeline(DataPipeline, Vocab):
             token_ids = token_ids[::-1]
         return token_ids + [self._eos_id]
 
-    def recover(self, input):
+    def decode(self, input):
         """ Recover one data sample.
 
         Args:
@@ -127,10 +147,4 @@ class TextDataPipeline(DataPipeline, Vocab):
         token_list = Vocab.map_id_to_token(self, input)
         if self._reverse_sequence:
             token_list = token_list[::-1]
-        if self._subtokenizer is None:
-            output = " ".join(token_list)
-        else:
-            output = self._subtokenizer.detokenize(token_list, return_str=True)
-        if self._tokenizer:
-            output = self._tokenizer.detokenize(output, return_str=True)
-        return output
+        return self.postprocess(" ".join(token_list))
