@@ -24,6 +24,7 @@ import numpy
 import six
 import tensorflow as tf
 from absl import logging
+from scipy.io import wavfile
 
 from neurst.data.audio import FeatureExtractor, build_feature_extractor
 from neurst.data.dataset_utils import load_tfrecords, take_one_record
@@ -31,17 +32,7 @@ from neurst.data.datasets import Dataset, TFRecordDataset, register_dataset
 from neurst.data.datasets.text_gen_dataset import TextGenDataset
 from neurst.utils.compat import DataStatus
 from neurst.utils.flags_core import Flag, ModuleFlag
-from neurst.utils.misc import to_numpy_or_python_type
-
-try:
-    import soundfile
-except (ImportError, OSError):
-    pass
-
-try:
-    from pydub import AudioSegment
-except ImportError:
-    pass
+from neurst.utils.misc import temp_download, to_numpy_or_python_type
 
 _languages = ["en", "de", "fr", "es", "it", "nl", "pt",
               "ro", "ru", "ar", "cs", "fa", "tr", "vi", "zh"]
@@ -71,18 +62,11 @@ class RawAudioDataset(Dataset):
         self._translations = None
         self._feature_extractor = build_feature_extractor(args)
         try:
-            import soundfile
-            _ = soundfile
-        except ImportError:
-            raise ImportError("Please install soundfile with: pip3 install soundfile")
-        except OSError:
-            raise OSError("sndfile library not found. Please install with: "
-                          "apt-get install libsndfile1")
-        try:
-            from pydub import AudioSegment
-            _ = AudioSegment
-        except ImportError:
-            raise ImportError("Please install pydub with: pip3 install pydub")
+            import sox
+            self._sox_transformer = sox.Transformer()
+            self._sox_transformer.set_output_format(rate=16000, file_type="wav")
+        except (ImportError, ModuleNotFoundError):
+            self._sox_transformer = None
         excluded_file = args["excluded_file"]
         self._excluded_str = None
         if excluded_file is not None:
@@ -177,24 +161,38 @@ class RawAudioDataset(Dataset):
             assert (file is None) ^ (fileobj is None), (
                 "Only one of `file` and `fileobj` should be provided.")
             mode = mode.lower()
-            if mode in ["flac", "wav"]:
-                sig, rate = soundfile.read(file or fileobj, dtype='float32')
-            elif mode == "mp3":  # need to re-sample and convert
-                tmp_wav = os.path.join(os.path.dirname(__file__), f"_tmp{time.time()}.wav")
-                mp3 = AudioSegment.from_file(file or fileobj, "mp3")
-                mp3.set_frame_rate(16000).export(tmp_wav, format="wav")
-                sig, rate = soundfile.read(tmp_wav, dtype="float32")
+            if mode == "wav":
+                rate, sig = wavfile.read(file or fileobj)
+            elif mode in ["mp3", "flac"]:  # need to re-sample and convert
+                if self._sox_transformer is None:
+                    raise RuntimeError("Please install sox environment: \n"
+                                       "\tapt-get install sox libavcodec-extra libsox-fmt-mp3\n"
+                                       "\tpip3 install sox")
+                if fileobj is None:
+                    fileobj = tf.io.gfile.GFile(file, "rb")
+                tmp_wav = os.path.join(os.path.dirname(__file__), f"_tmp{time.time()}." + mode)
+                with open(tmp_wav, "wb") as fw:
+                    fw.write(fileobj.read())
+                rate = 16000
+                sig = self._sox_transformer.build_array(input_filepath=tmp_wav)
+                if file is not None:
+                    fileobj.close()
+                tf.io.gfile.remove(tmp_wav)
             else:
                 raise NotImplementedError
+        if sig.dtype not in ["float32", "int16"]:
+            raise NotImplementedError(f"Not supported audio signal type {sig.dtype}.")
         if self._feature_extractor is not None:
             sig = self._feature_extractor(sig, rate)
+        elif sig.dtype == "int16":
+            sig = numpy.array(sig) / 32768.
         if numpy.any(numpy.isinf(sig)) or numpy.any(numpy.isnan(sig)):
             return None
         return sig
 
     @staticmethod
     def _pack_example_as_dict(audio, transcript=None, translation=None,
-                              src_lang=None, trg_lang=None):
+                              src_lang=None, trg_lang=None, **kwargs):
         example = {"audio": audio, "uuid": str(uuid.uuid4())}
         if transcript is not None:
             assert src_lang is not None
@@ -204,6 +202,8 @@ class RawAudioDataset(Dataset):
             assert trg_lang is not None
             example["translation"] = translation
             example["trg_lang"] = trg_lang
+        for k, v in kwargs.items():
+            example[k] = v
         return example
 
     @property
@@ -260,6 +260,8 @@ class AudioTFRecordDataset(TFRecordDataset, TextGenDataset):
         super(AudioTFRecordDataset, self).__init__(args)
         self._feature_key = args["feature_key"]
         self._transcript_key = args["transcript_key"]
+        if self._data_path.startswith("http"):
+            self._data_path = temp_download(self._data_path)
         example = take_one_record(self._data_path)
         if len(example.features.feature[self._feature_key].float_list.value) > 0:
             self._audio_is_extracted = True
@@ -272,7 +274,7 @@ class AudioTFRecordDataset(TFRecordDataset, TextGenDataset):
         elif len(example.features.feature[self._transcript_key].int64_list.value) > 0:
             self._transcript_is_projected = True
         else:
-            raise ValueError
+            self._transcript_is_projected = False
         if not hasattr(self, "_audio_is_extracted"):
             raise ValueError(f"Fail to read {self._data_path}")
 
@@ -378,6 +380,8 @@ class AudioTripleTFRecordDataset(TFRecordDataset, TextGenDataset):
         self._feature_key = args["feature_key"]
         self._transcript_key = args["transcript_key"]
         self._translation_key = args["translation_key"]
+        if self._data_path.startswith("http"):
+            self._data_path = temp_download(self._data_path)
         example = take_one_record(self._data_path)
         if len(example.features.feature[self._feature_key].float_list.value) > 0:
             self._audio_is_extracted = True
@@ -390,13 +394,13 @@ class AudioTripleTFRecordDataset(TFRecordDataset, TextGenDataset):
         elif len(example.features.feature[self._transcript_key].int64_list.value) > 0:
             self._transcript_is_projected = True
         else:
-            raise ValueError
+            self._transcript_is_projected = False
         if len(example.features.feature[self._translation_key].bytes_list.value) > 0:
             self._translation_is_projected = False
         elif len(example.features.feature[self._translation_key].int64_list.value) > 0:
             self._translation_is_projected = True
         else:
-            raise ValueError
+            self._translation_is_projected = False
         if not hasattr(self, "_audio_is_extracted"):
             raise ValueError(f"Fail to read {self._data_path}")
         self._transcripts = None
