@@ -9,6 +9,7 @@ from absl import logging
 
 from neurst.data.text import register_tokenizer
 from neurst.data.text.tokenizer import Tokenizer
+from neurst.utils.misc import temp_download
 
 RESERVED_TOKENS = []
 
@@ -435,6 +436,8 @@ class Subtokenizer(Tokenizer):
         if isinstance(codes, list):
             subtokens = codes
         else:
+            if codes.startswith("http://") or codes.startswith("https://"):
+                codes = temp_download(codes)
             with tf.io.gfile.GFile(codes, mode="r") as f:
                 subtokens = [line for line in f]
         for subtoken in subtokens:
