@@ -22,7 +22,7 @@ from neurst.data.dataset_utils import map_data_for_keras
 from neurst.data.datasets.multiple_dataset import MultipleDataset
 from neurst.exps import BaseExperiment, register_exp
 from neurst.models.model_utils import summary_model_variables
-from neurst.optimizers import OPTIMIZER_REGISTRY_NAME, build_optimizer
+from neurst.optimizers import OPTIMIZER_REGISTRY_NAME, build_optimizer, controlling_optimizer
 from neurst.optimizers.schedules import LR_SCHEDULE_REGISTRY_NAME, build_lr_schedule
 from neurst.sparsity.pruning_optimizer import create_pruning_optimizer
 from neurst.sparsity.pruning_schedule import PolynomialDecay, PruningSchedule, build_pruning_schedule
@@ -44,6 +44,7 @@ class Trainer(BaseExperiment):
         super(Trainer, self).__init__(**kwargs)
         self._tb_log_dir = args["tb_log_dir"]
         self._train_steps = args["train_steps"]
+        self._train_epochs = args["train_epochs"]
         self._summary_steps = args["summary_steps"]
         self._save_checkpoint_steps = args["save_checkpoint_steps"]
         self._checkpoints_max_to_keep = args["checkpoints_max_to_keep"]
@@ -82,6 +83,8 @@ class Trainer(BaseExperiment):
             else:
                 self._optimizer = build_optimizer(args, clipnorm=self._clip_norm, clipvalue=self._clip_value)
             assert self._optimizer is not None, "optimizer parameters must be provided for training."
+            self._optimizer = controlling_optimizer(self._optimizer, args["optimizer_controller"],
+                                                    args["optimizer_controller_args"])
         self._validator = build_validator(args)
         self._experimental_count_batch_num = args["experimental_count_batch_num"]
         self._freeze_variables = args["freeze_variables"]
@@ -100,6 +103,8 @@ class Trainer(BaseExperiment):
                        default=PolynomialDecay.__name__),
             Flag("tb_log_dir", dtype=Flag.TYPE.STRING, default=None,
                  help="The path to store tensorboard summary, or `model_dir`/train by default."),
+            Flag("train_epochs", dtype=Flag.TYPE.INTEGER, default=None,
+                 help="The number of times that the training process scans the whole data."),
             Flag("train_steps", dtype=Flag.TYPE.INTEGER, default=10000000,
                  help="The maximum steps for training loop."),
             Flag("summary_steps", dtype=Flag.TYPE.INTEGER, default=200,
@@ -133,15 +138,19 @@ class Trainer(BaseExperiment):
             Flag("nopruning_variable_pattern", dtype=Flag.TYPE.STRING, default=None,
                  help="The regular expression that indicates the variables will NOT be pruned "
                       "(will take effect if `pruning_variable_pattern`=None)."),
+            Flag("optimizer_controller", dtype=Flag.TYPE.STRING, default=None,
+                 help="An optimizer wrapper controlling the specific operations during model training."),
+            Flag("optimizer_controller_args", dtype=Flag.TYPE.STRING, default=None,
+                 help="A dict of parameters for optimizer controller."),
         ]
 
     def _restore_ckpt_or_pretrain(self):
         """ restoring checkpoint from model_dir or pretrain_model dir. """
         stat = restore_checkpoint_if_possible(self.model, self.model_dir)
-        continue_training = False
+        continue_training_from = None
         if stat:
             logging.info(f"Successfully restoring checkpoint from model_dir={self.model_dir}")
-            continue_training = True
+            continue_training_from = self.model_dir
         else:
             logging.info(f"No checkpoint restored from model_dir={self.model_dir}")
             if self._pretrain_model:
@@ -157,9 +166,11 @@ class Trainer(BaseExperiment):
                         logging.info("NOTE THAT, one must first check the variable names in this checkpoint, "
                                      "otherwise no variables will be restored.")
                         restore_checkpoint_if_possible(self.model, pt, var_name_pattern=pt_varname)
+                        if len(self._pretrain_model) == 1 and pt_varname is None:
+                            continue_training_from = pt
 
-        if self._initial_global_step is None and continue_training:
-            _step = compat.hack_global_step(self.model_dir)
+        if self._initial_global_step is None and continue_training_from:
+            _step = compat.hack_global_step(continue_training_from)
             if _step:
                 compat.register_initial_step(_step or 0)  # must do this before creating optimizer and training
                 logging.info(f"Restored initial global step={_step}")
@@ -283,11 +294,22 @@ class Trainer(BaseExperiment):
                 cnt += 1
             logging.info(f"Total {cnt} batches per EPOCH.")
 
-        history = keras_model.fit(
-            map_data_for_keras(tfds.repeat()),
-            initial_epoch=0,
-            epochs=1,
-            steps_per_epoch=self._train_steps,  # * args["update_cycle"],
-            verbose=2,
-            callbacks=training_callbacks)
+        if self._train_epochs:
+            logging.info(f"Training for {self._train_epochs} epochs.")
+            history = keras_model.fit(
+                map_data_for_keras(tfds),
+                initial_epoch=0,
+                epochs=self._train_epochs,
+                verbose=2,
+                callbacks=training_callbacks)
+        else:
+            logging.info(f"Training for {self._train_steps} steps.")
+            history = keras_model.fit(
+                map_data_for_keras(tfds.repeat()),
+                initial_epoch=0,
+                epochs=1,
+                steps_per_epoch=self._train_steps,
+                verbose=2,
+                callbacks=training_callbacks)
+
         logging.info(history.history)
