@@ -46,11 +46,17 @@ class Task(object):
     def create_inputs(self, mode):
         """ Creates keras input placeholders. """
         dtypes, signatures = self.inputs_signature(mode)
-        inps = dict()
-        for name in dtypes:
-            inps[name] = tf.keras.layers.Input(tuple(signatures[name][1:]),
-                                               dtype=dtypes[name], name=name)
-        return inps
+        if isinstance(dtypes, dict):
+            inps = dict()
+            for name in dtypes:
+                inps[name] = tf.keras.layers.Input(tuple(signatures[name][1:]),
+                                                   dtype=dtypes[name], name=name)
+            return inps
+        else:
+            inps = []
+            for d, s in zip(tf.nest.flatten(dtypes), tf.nest.flatten(signatures)):
+                inps.append(tf.keras.layers.Input(tuple(s[1:]), dtype=d))
+            return tf.nest.pack_sequence_as(dtypes, inps)
 
     @abstractmethod
     def inputs_signature(self, mode) -> Tuple[dict, dict]:
@@ -68,7 +74,7 @@ class Task(object):
             according to this task. """
         raise NotImplementedError("Task must implement the `get_data_preprocess_fn` method.")
 
-    def get_data_postprocess_fn(self, mode) -> callable:
+    def get_data_postprocess_fn(self, data_status, **kwargs) -> callable:
         """ Returns a callable function that postprocess the data sample
             according to this task. """
         raise NotImplementedError
@@ -92,7 +98,7 @@ class Task(object):
         return [x for x in COMMON_DATA_ARGS]
 
     @abstractmethod
-    def build_model(self, args, name=None):
+    def build_model(self, args, name=None, **kwargs):
         """Build a new model instance."""
         raise NotImplementedError("Task must implement the `build_model` method.")
 
