@@ -13,9 +13,13 @@
 # limitations under the License.
 import argparse
 import copy
+import importlib
 import json
+import os
+import traceback
 from collections import namedtuple
 
+import tensorflow as tf
 from absl import logging
 
 from neurst.utils.configurable import deep_merge_dict, load_from_config_path, yaml_load_checking
@@ -200,6 +204,48 @@ DEFAULT_CONFIG_FLAG = Flag(name="config_paths", dtype=Flag.TYPE.STRING, multiple
                                 "Setting a key in these files is equivalent to "
                                 "setting the FLAG value with the same name.")
 
+EXTRA_IMPORT_LIB = Flag(name="include", dtype=Flag.TYPE.STRING, multiple=True,
+                        help="The extra python path to be included and imported.")
+
+
+def add_extra_includes():
+    arg_parser = argparse.ArgumentParser()
+    EXTRA_IMPORT_LIB.define(arg_parser)
+    parsed, _ = arg_parser.parse_known_args()
+    include = parsed.include
+    if include is None:
+        return
+    for path in include:
+        if not os.path.isdir(path):
+            try:
+                importlib.import_module(path)
+                logging.info(f"[INFO] import user package {path}")
+            except (RuntimeError, ImportError, tf.errors.OpError) as e:
+                logging.info(traceback.format_exc(e))
+                logging.info(f"WARNING: fail to import {path}")
+            continue
+        for file in os.listdir(path):
+            if not file.startswith('_') and not file.startswith('.') and file.endswith('.py'):
+                module_name = file[:file.find('.py')] if file.endswith('.py') else file
+                src_file = os.path.join(path, file)
+                with tf.io.gfile.GFile(src_file) as fp:
+                    should_skip = True
+                    for line in fp:
+                        if line.strip().startswith("@register"):
+                            should_skip = False
+                            break
+                    if should_skip:
+                        logging.info(f"[INFO] skip {src_file}")
+                        continue
+                trg_file = os.path.join(os.path.dirname(__file__), "userdef/" + file)
+                tf.io.gfile.copy(src_file, trg_file, overwrite=True)
+                try:
+                    importlib.import_module("neurst.utils.userdef." + module_name)
+                    logging.info(f"[INFO] import user-defined {src_file}")
+                except (RuntimeError, ImportError, tf.errors.OpError) as e:
+                    logging.info(traceback.format_exc(e))
+                    logging.info(f"WARNING: fail to import {src_file}")
+
 
 def define_flags(flag_list: list, arg_parser=None, with_config_file=True) -> argparse.ArgumentParser:
     """ Defines the root module name.
@@ -211,6 +257,7 @@ def define_flags(flag_list: list, arg_parser=None, with_config_file=True) -> arg
 
     Returns: The argument parser.
     """
+    add_extra_includes()
     if arg_parser is None:
         arg_parser = argparse.ArgumentParser()
     if with_config_file:
@@ -442,6 +489,27 @@ def extend_define_and_parse(flag_name, args, remaining_argv, backend="tf"):
     return args, remaining_argv
 
 
+def _handle_too_long_verbosity(indent, key, val, default, help_txt):
+    text = indent + key + ": "
+    if isinstance(val, list) and len(val) > 10:
+        text += str(val[:10] + ["....."])
+    elif isinstance(val, dict):
+        new_val = {}
+        for k, v in val.items():
+            if isinstance(v, list) and len(v) > 10:
+                new_val[k] = v[:10] + ["......"]
+            else:
+                new_val[k] = v
+        text += json.dumps(new_val)
+    else:
+        text += f"{val}"
+    text += "     # "
+    if default is not None:
+        text += f"(default: {default}) "
+    text += help_txt
+    logging.info(text)
+
+
 def verbose_flags(flag_list, args, remaining_argv, backend="tf"):
     logging.info("==========================================================================")
     logging.info("Parsed all matched flags: ")
@@ -460,15 +528,15 @@ def verbose_flags(flag_list, args, remaining_argv, backend="tf"):
                     for ff in REGISTRIES[backend][f.module_name][verbose_args[f.cls_key]].class_or_method_args():
                         if isinstance(ff, Flag):
                             if ff.name in verbose_args[f.params_key]:
-                                logging.info(f"   {ff.name}: {verbose_args[f.params_key][ff.name]}"
-                                             f"     # (default: {ff.default}) {ff.help}")
+                                _handle_too_long_verbosity("   ", ff.name, verbose_args[f.params_key][ff.name],
+                                                           ff.default, ff.help)
                         else:
                             if ff.cls_key in verbose_args[f.params_key]:
                                 logging.info(f"   {ff.cls_key}: {verbose_args[f.params_key][ff.cls_key]}")
                             if ff.params_key in verbose_args[f.params_key]:
-                                logging.info(f"   {ff.params_key}: "
-                                             f"{json.dumps(verbose_args[f.params_key][ff.params_key])}"
-                                             f"     # {ff.help}")
+                                _handle_too_long_verbosity("   ", ff.params_key,
+                                                           verbose_args[f.params_key][ff.params_key],
+                                                           None, ff.help)
                 else:
                     logging.info(f" {f.params_key}: {json.dumps(verbose_args[f.params_key])}")
             verbose_args.pop(f.cls_key, None)
@@ -487,6 +555,8 @@ def verbose_flags(flag_list, args, remaining_argv, backend="tf"):
                 if text and len(text) > 0:
                     logging.info("  {}".format(" ".join(text)))
                 text = []
+            if text is None:
+                continue
             text.append(arg)
         if text and len(text) > 0:
             logging.info("  {}".format(" ".join(text)))
