@@ -11,8 +11,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import functools
+import inspect
 import multiprocessing
 import os
+import tempfile
 from distutils.version import LooseVersion
 from urllib.request import urlretrieve
 
@@ -21,16 +24,41 @@ import tensorflow as tf
 from absl import logging
 
 
-def deprecated(obj):
+def deprecated(substitution):
     """This is a decorator which can be used to mark functions or classes
     as deprecated. It will result in a warning being emmitted
     when the function/class is used."""
 
-    def new_obj(*args, **kwargs):
-        logging.info("WARNING: Calling deprecated function/class: %s." % obj.__name__)
-        return obj(*args, **kwargs)
+    cls_warn = "Call to deprecated class `{name}`. "
+    func_warn = "Call to deprecated function `{name}`. "
+    if isinstance(substitution, str):
 
-    return new_obj
+        def decorator(func):
+            warn_info = cls_warn if inspect.isclass(func) else func_warn
+            warn_info += "Please use `{substitution}` instead."
+
+            @functools.wraps(func)
+            def new_func(*args, **kwargs):
+                logging.warning(warn_info.format(name=func.__name__, substitution=substitution))
+                return func(*args, **kwargs)
+
+            return new_func
+
+        return decorator
+
+    elif inspect.isclass(substitution) or inspect.isfunction(substitution):
+        func = substitution
+        warn_info = cls_warn if inspect.isclass(func) else func_warn
+
+        @functools.wraps(func)
+        def new_func(*args, **kwargs):
+            logging.warning(warn_info.format(name=func.__name__))
+            return func(*args, **kwargs)
+
+        return new_func
+
+    else:
+        raise TypeError(repr(type(substitution)))
 
 
 def flatten_string_list(arg):
@@ -43,7 +71,7 @@ def flatten_string_list(arg):
     """
     if arg is None:
         return None
-    return [c.strip() for cs in tf.nest.flatten(arg) for c in cs.split(",")]
+    return [c.strip() for cs in tf.nest.flatten(arg) for c in cs.split(",") if c.strip()]
 
 
 class DummyContextManager(object):
@@ -167,6 +195,16 @@ def download_with_tqdm(url, filename):
         urlretrieve(url, filename, reporthook=t.update_to, data=None)
 
 
+def temp_download(url):
+    tmpfile = tempfile.NamedTemporaryFile("w", delete=False)
+    download_with_tqdm(url, tmpfile.name)
+    inmemory_name = "ram://" + os.path.basename(tmpfile.name)
+    with tf.io.gfile.GFile(inmemory_name, "wb") as fw, tf.io.gfile.GFile(tmpfile.name, "rb") as fp:
+        fw.write(fp.read())
+    os.remove(tmpfile.name)
+    return inmemory_name
+
+
 def assert_equal_numpy(tensor_a, tensor_b, epsilon=1e-6):
     """
 
